'use strict';

var fs = require('fs');
var path = require('path');
var vfs = require('vinyl-fs');
var combiner = require('stream-combiner2').obj;
var through2 = require('through2').obj;
var $ = require('gulp-load-plugins')();
var CheckDiffSize = require('./CheckDiffSize.js');

var setProp = function setProp(obj, key, val) {
	if (obj[key] == null) {
		obj[key] = val;
	}
};

var isolatedStream = function isolatedStream(src, stats, destPathTheme, base_patch, theme, options) {
	return combiner(vfs.src(src, {
		read: false
	}), through2(function (file, enc, cb) {
		fs.lstat(file.path, function (err, stats) {
			if (err !== null) {
				return cb(err, null);
			}

			if (stats.isSymbolicLink()) {
				if (options.debug) {
					console.log('Read SymbolicLink: ' + file.path + ' => ' + fs.realpathSync(file.path));
				}
				fs.readFile(fs.realpathSync(file.path), function (err, data) {
					if (err !== null) {
						return cb(err, file);
					}

					if (data.length != 0) {
						file.contents = data;
						return cb(null, file);
					} else {
						return cb(null, null);
					}
				});
			} else {
				fs.readFile(file.path, function (err, data) {
					if (err !== null) {
						return cb(err, file);
					}

					if (data.length != 0) {
						file.contents = data;
						return cb(null, file);
					} else {
						return cb(null, null);
					}
				});
			}
		});
	}), stats.checkSrc(), $.if(function (file) {
		return (/\.css$/.test(file.basename)
		);
	}, $.postcss([require('postcss-remove-prefixes')()])), $.if(function (file) {
		return (/\.css$/.test(file.basename)
		);
	}, $.postcss([require('autoprefixer')(options.autoprefixer), require('postcss-csso')(options.csso)])), $.if(function (file) {
		return (/\.js$/.test(file.basename)
		);
	}, $.uglify(options.uglify)), stats.writeDiff(destPathTheme), vfs.dest(destPathTheme)) /*.on(
                                                                                        'finish',
                                                                                        stats.onFinish()
                                                                                        )*/.on('error', $.notify.onError({
		title: 'MagentoMinify',
		message: "Error: <%= error.stack %>"
	}));
};

var procesedTheme = function procesedTheme(base_patch, theme, options) {
	var stats = new CheckDiffSize();
	var destPathRequirejs = path.resolve(base_patch + '/pub/static/_requirejs/frontend/' + theme.vendor + '/' + theme.theme + '/' + theme.lang + '/');
	var destPathTheme = path.resolve(base_patch + '/pub/static/frontend/' + theme.vendor + '/' + theme.theme + '/' + theme.lang + '/');

	if (options.debug) {
		console.log(destPathTheme, '\n', destPathRequirejs);
	}

	// minify require.js configs
	combiner(vfs.src([destPathRequirejs + '/**/*.js']), stats.checkSrc(), $.uglify(options.uglify), stats.writeDiff(destPathRequirejs), vfs.dest(destPathRequirejs)).on('error', $.notify.onError({
		title: 'MagentoMinify',
		message: "Error: <%= error.stack %>"
	}));

	combiner(vfs.src([destPathTheme + '/**/*.{css,js}', '!' + destPathTheme + '/css/docs/*'], {
		read: false
	})).on('data', function (file) {
		setTimeout(function () {
			isolatedStream(file.path, stats, destPathTheme, base_patch, theme, options);
		}, 0);
		this.emit('end');
	});
};

module.exports = function (base_patch, themes, options) {

	options = options || {};
	setProp(options, 'debug', false);
	setProp(options, 'uglify', {
		output: { comments: false },
		compress: {
			warnings: false,
			conditionals: true,
			unused: true,
			comparisons: true,
			sequences: true,
			dead_code: true,
			evaluate: true,
			if_return: true,
			join_vars: true,
			negate_iife: false,
			drop_console: true,
			unsafe: true } });
	setProp(options, 'autoprefixer', {
		browsers: ['Chrome >= 35', 'Firefox >= 38', 'Edge >= 12', 'Explorer >= 10', 'iOS >= 8', 'Safari >= 8', 'Android 2.3', 'Android >= 4', 'Opera >= 12'] });
	setProp(options, 'csso', {});

	if (!Array.isArray(themes)) {
		themes = [themes];
	}

	themes.forEach(function (el) {
		setTimeout(function () {
			procesedTheme(base_patch, el, options);
		}, 0);
	});
};